from setuptools import find_packages, setup

setup(
    name='stone_color',
    packages=find_packages(),
    version='0.3.0',
    long_description='Stone_color is a style library for Python that can perform text formatting, coloring, tables, and loading bars.',
    description='Text formatting, coloring and tables library',
    keywords=["color", "cli", "table", "terminal-colors", "ansi", "rgb"],
    author='aniko33',
)
